@@ -8,7 +8,11 @@ num2pal <- function(x, n = 5,
 					   legend.scientific = FALSE,
 					   legend.digits = NA,
 					   colorNA = "#FF1414",
-					   legend.NA.text = "Missing") {
+					   legend.NA.text = "Missing",
+					   alpha=1,
+					   text_separator = "to",
+					   text_less_than = "Less than",
+					   text_or_more = "or more") {
 	
 	if (length(x)==1) stop("Statistical numerical variable only contains one value. Please use a constant value instead.")
 	# create intervals and assign colors
@@ -32,26 +36,45 @@ num2pal <- function(x, n = 5,
 	
 	
 	# map palette
-	mc <- brewer.pal.info[palette, "maxcolors"]
-	if (auto.palette.mapping) {
-		pal.div <- (brewer.pal.info[palette, "category"]=="div")
-
-		colpal <- colorRampPalette(revPal(brewer.pal(mc, palette)))(101)
-
-		ids <- if (pal.div) {
-			map2divscaleID(breaks, contrast=contrast)
+	if (palette[1] %in% rownames(brewer.pal.info)) {
+		mc <- brewer.pal.info[palette, "maxcolors"]
+		if (auto.palette.mapping) {
+			pal.div <- (brewer.pal.info[palette, "category"]=="div")
+			
+			colpal <- colorRampPalette(revPal(brewer.pal(mc, palette)))(101)
+			
+			ids <- if (pal.div) {
+				map2divscaleID(breaks, n=101, contrast=contrast)
+			} else {
+				map2seqscaleID(breaks, n=101, contrast=contrast)
+			}
+			
+			legend.palette <- colpal[ids]
+			if (any(ids<51) && any(ids>51)) {
+				ids.neutral <- min(ids[ids>=51]-51) + 51
+				legend.neutral.col <- colpal[ids.neutral]
+			} else {
+				legend.neutral.col <- colpal[ids[round(((length(ids)-1)/2)+1)]]
+			}
+			
 		} else {
-			map2seqscaleID(breaks, contrast=contrast)
+			if (nbrks-1 > mc) {
+				legend.palette <- colorRampPalette(revPal(brewer.pal(mc, palette)))(nbrks-1)
+			} else legend.palette <- revPal(brewer.pal(nbrks-1, palette))
+			legend.neutral.col <- legend.palette[round(((length(legend.palette)-1)/2)+1)]
 		}
-		
-		legend.palette <- colpal[ids]
 	} else {
-		if (nbrks-1 > mc) {
-			legend.palette <- colorRampPalette(revPal(brewer.pal(mc, palette)))(nbrks-1)
-		} else legend.palette <- revPal(brewer.pal(nbrks-1, palette))
+		legend.palette <- rep(palette, length.out=nbrks-1)
+		legend.neutral.col <- legend.palette[1]
 	}
 	
-	cols <- findColours(q, legend.palette)
+	legend.palette <- get_alpha_col(legend.palette, alpha)
+	legend.neutral.col <- get_alpha_col(legend.neutral.col, alpha)
+	colorNA <- get_alpha_col(colorNA, alpha)
+	
+	
+	ids <- findCols(q)
+	cols <- legend.palette[ids]
 	anyNA <- any(is.na(cols))
 	if (anyNA) {
 		cols[is.na(cols)] <- colorNA
@@ -73,9 +96,9 @@ num2pal <- function(x, n = 5,
 		} else {
 			breaks.printed <- fancy_breaks(breaks, dp=legend.digits) 
 			breaks.printed[breaks==-Inf] <- ""
-			legend.labels <- paste(breaks.printed[-nbrks], breaks.printed[-1], sep=" to ")
-			if (breaks[1]==-Inf) legend.labels[1] <- paste("Less than", breaks.printed[2])
-			if (breaks[nbrks]==Inf) legend.labels[nbrks-1] <- paste(breaks.printed[nbrks-1], "or more")
+			legend.labels <- paste(breaks.printed[-nbrks], breaks.printed[-1], sep = paste0(" ", text_separator, " "))
+			if (breaks[1]==-Inf) legend.labels[1] <- paste(text_less_than, breaks.printed[2])
+			if (breaks[nbrks]==Inf) legend.labels[nbrks-1] <- paste(breaks.printed[nbrks-1], text_or_more)
 		}
 		
 	} else {
@@ -87,7 +110,7 @@ num2pal <- function(x, n = 5,
 		legend.labels <- c(legend.labels, legend.NA.text)
 	}
 	
-	list(cols=cols, legend.labels=legend.labels, legend.palette=legend.palette, breaks=breaks)
+	list(cols=cols, legend.labels=legend.labels, legend.palette=legend.palette, breaks=breaks, legend.neutral.col = legend.neutral.col)
 }
 
 
