@@ -9,9 +9,12 @@ process_fill_vector <- function(x, g, gt, tiny) {
 		palette <- if (is.null(g$palette)) ifelse(nlevels(x)>8, "Set3", "Dark2") else g$palette
 		colsLeg <- cat2pal(x,
 						   palette = palette,
+						   contrast = g$contrast,
 						   colorNA = g$colorNA,
+						   legend.labels=g$labels,
 						   legend.NA.text = textNA,
-						   max_levels=g$max.categories)
+						   max_levels=g$max.categories,
+						   alpha=g$alpha)
 		fill.breaks <- NA
 	} else {
 		palette <- if (is.null(g$palette)) "RdYlGn" else g$palette
@@ -22,7 +25,11 @@ process_fill_vector <- function(x, g, gt, tiny) {
 						   colorNA=g$colorNA, 
 						   legend.scientific=gt$legend.scientific,
 						   legend.digits=gt$legend.digits,
-						   legend.NA.text = textNA)
+						   legend.NA.text = textNA,
+						   alpha=g$alpha, 
+						   text_separator = g$text_separator,
+						   text_less_than = g$text_less_than,
+						   text_or_more = g$text_or_more)
 		fill.breaks <- colsLeg[[4]]
 	}
 	fill <- colsLeg[[1]]
@@ -45,7 +52,7 @@ process_fill_vector <- function(x, g, gt, tiny) {
 }
 
 
-process_fill <- function(data, g, gb, gt, gby) {
+process_fill <- function(data, g, gb, gt, gby, z) {
 	
 	npol <- nrow(data)
 	by <- data$GROUP_BY
@@ -59,19 +66,26 @@ process_fill <- function(data, g, gb, gt, gby) {
 	nx <- length(x)
 	
 	# check for direct color input
-	if (all(valid_colors(x))) {
+	is.colors <- all(valid_colors(x))
+	if (is.colors) {
+		x <- get_alpha_col(col2hex(x), g$alpha)
 		for (i in 1:nx) data[[paste("COLOR", i, sep="_")]] <- x[i]
 		x <- paste("COLOR", 1:nx, sep="_")
 	} else {
 		if (!all(x %in% shpcols)) stop("Fill argument neither colors nor valid variable names")
 	}
-	dt <- process_data(data[, x, drop=FALSE], by=by, free.scales=gby$free.scales.fill)
+	dt <- process_data(data[, x, drop=FALSE], by=by, free.scales=gby$free.scales.fill, is.colors=is.colors)
 	## output: matrix=colors, list=free.scales, vector=!freescales
 	
 	nx <- max(nx, nlevels(by))
 		
 	# return if data is matrix of color values
-	if (is.matrix(dt)) return(list(fill=dt, fill.alpha=g$alpha, xfill=rep(NA, nx)))
+	if (is.matrix(dt)) {
+		if (!is.colors) {
+			dt <- matrix(get_alpha_col(dt, g$alpha), ncol=ncol(dt))
+		}
+		return(list(fill=dt, xfill=rep(NA, nx), fill.lenged.title=rep(NA, nx)))	
+	} 
 	tiny <- areas_prop < g$thres.poly
 	if (all(tiny)) warning("all relative area sizes are below thres.poly")
 	if (is.list(dt)) {
@@ -98,10 +112,28 @@ process_fill <- function(data, g, gb, gt, gby) {
 		fill.breaks <- res$fill.breaks
 		fill.values <- lapply(split(dt, rep(1:nx, each=npol)), function(d)d[!tiny])
 	}
+	fill.legend.title <- if (is.na(g$title)[1]) x else g$title
+	fill.legend.z <- if (is.na(g$legend.z)) z else g$legend.z
+	fill.legend.hist.z <- if (is.na(g$legend.hist.z)) z+.5 else g$legend.hist.z
+	
+	if (g$legend.hist && is.na(g$legend.hist.title) && fill.legend.z>fill.legend.hist.z) {
+		# histogram is drawn between title and legend enumeration
+		fill.legend.hist.title <- fill.legend.title
+		fill.legend.title <- ""
+	} else if (g$legend.hist && !is.na(g$legend.hist.title)) {
+		fill.legend.hist.title <- g$legend.hist.title
+	} else fill.legend.hist.title <- ""
+	
 	list(fill=fill,
-		 fill.alpha=g$alpha,
 		 fill.legend.labels=fill.legend.labels,
 		 fill.legend.palette=fill.legend.palette,
-		 fill.legend.misc=list(values=fill.values, breaks=fill.breaks, alpha=g$alpha, lwd=gb$lwd, border.col=gb$col, border.alpha=gb$alpha),
-		 xfill=x)
+		 fill.legend.misc=list(lwd=gb$lwd, border.col=gb$col, border.alpha=gb$alpha),
+		 fill.legend.hist.misc=list(values=fill.values, breaks=fill.breaks),
+		 xfill=x,
+		 fill.legend.title=fill.legend.title,
+		 fill.legend.is.portrait=g$legend.is.portrait,
+		 fill.legend.hist=g$legend.hist,
+		 fill.legend.hist.title=fill.legend.hist.title,
+		 fill.legend.z=fill.legend.z,
+		 fill.legend.hist.z=fill.legend.hist.z)
 }
