@@ -1,5 +1,5 @@
-process_tm <- function(x, asp_ratio) {
-	fill <- NULL; xfill <- NULL
+process_tm <- function(x, asp_ratio, shp_info) {
+	fill <- NULL; xfill <- NULL; xraster <- NULL
 	## fill meta info
 	
 	## get tm_layout elements
@@ -23,6 +23,14 @@ process_tm <- function(x, asp_ratio) {
 	gridid <- which(names(x)=="tm_grid")[1]
 	gg <- x[[gridid]]
 	
+	## get credits element
+	creditsid <- which(names(x)=="tm_credits")[1]
+	gc <- x[[creditsid]]
+	
+	## get scale bar element
+	scaleid <- which(names(x)=="tm_scale")[1]
+	gsb <- x[[scaleid]]
+	
 	## get facets element
 	shape.id.orig <- which(names(x)=="tm_shape")
 	facet.id.orig <- which(names(x)=="tm_facets")
@@ -56,39 +64,26 @@ process_tm <- function(x, asp_ratio) {
 		}
 	}
 	
-# 	
-# 	facetids <- which(names(x)=="tm_facets")
-# 	if (length(facetids)) {
-# 		shape.id.orig <- which(names(x)[1:facetid]=="tm_shape")
-# 		gf.shp.id <- tail(shape.id.orig, 1)
-# 		gf <- x[[facetid]]
-# 		gf$shp_nr <- ifelse(!is.null(gf$by) && gf$free.coords, length(shape.id.orig), 0)
-# 		gf$shp_name <- x[[gf.shp.id]]$shp_name
-# 	} else {
-# 		gf <- tm_facets()$tm_facets 
-# 		gf$shp_nr <- 0
-# 		gf$shp_name <- ""
-# 	}
 	
 	## split x into gmeta and gbody
-	x <- x[!(names(x) %in% c("tm_layout", "tm_grid", "tm_facets"))]
+	x <- x[!(names(x) %in% c("tm_layout", "tm_grid", "tm_facets", "tm_credits", "tm_scale_bar"))]
 
 	n <- length(x)
 	
 	## split x into clusters
 	shape.id <- which(names(x)=="tm_shape")
-	if (shape.id[1] != 1) stop("First layers should be a tm_shape layer.")
+	if (shape.id[1] != 1) stop("First element should be a tm_shape element.")
 	y <- rep(0, n); y[shape.id] <- 1
 	cluster.id <- cumsum(y)
 	gs <- split(x, cluster.id)
 	
 	nlx <- sapply(gs, length)
-	if (any(nlx==1)) warning("Specify at least one layer next to tm_shape")
+	if (any(nlx==1)) warning("Specify at least one layer after each tm_shape")
 	
 	
-	#gs <- lapply(gs, function(gx) if (is.null(gx[["tm_borders"]])) gx + tm_borders() else gx)
 	## convert clusters to layers
-	gp <- lapply(gs, FUN=process_layers, gt, gf)
+	cnlx <- if (nshps==1) 0 else c(0, cumsum(nlx[1:(nshps-1)]-1))
+	gp <- mapply(FUN=process_layers, gs, cnlx, MoreArgs = list(gt=gt, gf=gf), SIMPLIFY = FALSE)
 	
 	## get by vector
 	data_by <- lapply(gp, function(i)i$data_by)
@@ -104,25 +99,13 @@ process_tm <- function(x, asp_ratio) {
 	
 
 	by_counts <- sapply(data_by, nlevels)
-	if (sum(by_counts>0)>1) {
-		by_counts_pos <- by_counts[by_counts!=0]
+	if (sum(by_counts>1)>1) {
+		by_counts_pos <- by_counts[by_counts>1]
 		if (any(by_counts_pos[-1]!=by_counts_pos[1])) stop("Number of facets defined by the 'by' argument of tm_facets are different for the groups.")
 	}
 
 
 
-	## determine maximal number of variables
-	
-# 	nx <- max(sapply(gp, function(x) {
-# 		max(length(x$varnames$by),
-# 			ifelse(is.matrix(x$fill), ncol(x$fill), 1),
-# 			ifelse(is.matrix(x$bubble.size), ncol(x$bubble.size), 1),
-# 			ifelse(is.matrix(x$bubble.col), ncol(x$bubble.col), 1),
-# 			ifelse(is.matrix(x$line.col), ncol(x$line.col), 1),
-# 			ifelse(is.matrix(x$line.lwd), ncol(x$line.lwd), 1),
-# 			ifelse(is.matrix(x$text), ncol(x$text), 1))
-# 	}))
-# 	
 	nx <- max(sapply(gp, function(x){
 		max(sapply(x$varnames, length))
 	}))
@@ -132,12 +115,18 @@ process_tm <- function(x, asp_ratio) {
 	names(gp) <- paste0("tmLayer", 1:length(gp))
 	
 	## get variable names (used for titles)
-	varnames <- process_varnames(gp, nx)
+	#varnames <- process_varnames(gp, nx)
+	
+	
+	## get by names
+	by_names_list <- lapply(gp, function(gpl) gpl$varnames$by)
+	by_names_specified <- !sapply(by_names_list, is.na)
 
+	by_names <- if (any(by_names_specified)) by_names_list[[which(by_names_specified)[1]]] else NA
 	## process grid
-	gmeta <- process_meta(gt, gf, gg, nx, varnames, asp_ratio)
+	gmeta <- c(process_meta(gt, gf, gg, gc, gsb, nx, by_names, asp_ratio), shp_info)
+	
 	## split into small multiples
-
 	gps <- split_tm(gp, nx, order_by)
 	scale <- gmeta$scale
 	gps <- mapply(function(x, i){
@@ -146,15 +135,10 @@ process_tm <- function(x, asp_ratio) {
 				lwd <- lwd * scale
 				
 				if (!is.null(fill)) {
-					#if (!is.null(data_by)) fill <- fill[i]
 					if (!is.na(xfill[1])) fill.legend.misc$lwd <- fill.legend.misc$lwd * scale
 				}
 
 				if (!is.null(bubble.size)) {
-# 					if (!is.null(data_by)) {
-# 						bubble.size <- bubble.size[i]
-# 						bubble.col <- bubble.col[i]
-# 					}
 					
 					bubble.size <- bubble.size * scale
 					bubble.border.lwd <- bubble.border.lwd * scale
@@ -166,10 +150,6 @@ process_tm <- function(x, asp_ratio) {
 				}
 				
 				if (!is.null(line.lwd)) {
-# 					if (!is.null(data_by)) {
-# 						line.lwd <- line.lwd[i]
-# 						line.col <- line.col[i]
-# 					}
 					
 					line.lwd <- line.lwd * scale
 					line.col.legend.misc$line.legend.lwd <- line.col.legend.misc$line.legend.lwd * scale
@@ -177,14 +157,20 @@ process_tm <- function(x, asp_ratio) {
 				}
 				
 				if (!is.null(text)) {
-					text.cex <- text.cex * scale
+					text.size <- text.size * scale
 				}
+
+
+				if (!is.null(raster)) {
+					if (!is.na(xraster[1])) raster.legend.misc$lwd <- raster.legend.misc$lwd * scale
+				}
+
 			})
 		})
 		
 		x$tm_layout <- gmeta
 		x$tm_layout$title <- x$tm_layout$title[i]
-		x$tm_layout$legend.titles <- sapply(x$tm_layout$legend.titles, function(x)x[i])
+		#x$tm_layout$legend.titles <- sapply(x$tm_layout$legend.titles, function(x)x[i])
 		x
 	}, gps, 1:nx, SIMPLIFY=FALSE)
 	
